/*
Copyright 2006 Extrarius (extrarius@extrarius.name)
Redistribution and use without modification are permitted.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*
This program is meant to fix the sourge 'engine.dll' used by
'Vampire: The Masquerade Bloodlines'(TM) so that it will work
on machines with more than 2GB of RAM.

The game must be patched to version 1.20 for this to work.

There is no guarantee that it will function correctly.
*/
#include <stdio.h>
#include <memory.h>

void output(char *message)
{
	fputs(message, stdout);
}

void usage(void)
{
	output("Usage: fix_vamp <engine.dll path>\n");
}

int patch_vamp(char *dll_name)
{
	FILE *dll = NULL;
	/*The value the bytes should have*/
	char correct_bytes[16] = {
		0xC8, 0x2B, 0xB4, 0x20, 0x7D, 0x12, 0x50, 0x68,
		0x98, 0xCC, 0x1A, 0x20, 0xE8, 0x0F, 0xFE, 0xFF
	};
	/*The buffer to load the bytes in*/
	char buffer[16];
	/*Open the file*/
	dll = fopen(dll_name, "r+b");
	if(dll == NULL)
	{
		/*Open failed, print error and usage*/
		output("Error: Unable to open file '");
		output(dll_name);
		output("'!\n");
		usage();
		return 1;
	}
	/*Move to the 'hex line' with the opcode to change*/
	if(fseek(dll, 0xFA490, SEEK_SET) != 0)
	{
		//Unable to move to that location, print error and usage
		fclose(dll);
		output("Error: Unable to seek to the proper position in '");
		output(dll_name);
		output("'!\n");
		usage();
		return 1;
	}
	/*Read the bytes at that location to ensure they match*/
	if(fread(buffer, 1, 16, dll) != 16)
	{
		/*Unable to read enough bytes, print error and usage*/
		fclose(dll);
		output("Error: Unable to read a signature from '");
		output(dll_name);
		output("'!\n");
		usage();
		return 1;
	}
	/*Ensure the bytes match*/
	if(memcmp(correct_bytes, buffer, 16) != 0)
	{
		/*The bytes do not match, print error and usage*/
		fclose(dll);
		output("Error: Expected signature was not found in '");
		output(dll_name);
		output("'!\n");
		usage();
		return 1;
	}
	/*
	Make the required changes:
	  jge works on signed numbers, but the numbers are actually unsigned
	  Thus, changing it to jae, which works on unsigned numbers, fixes it
	  7D to 73 changes 'jge' to 'jae'
	*/
	buffer[4] = 0x73;
	/*Move to the 'hex line' with the opcode to change*/
	if(fseek(dll, 0xFA490, SEEK_SET) != 0)
	{
		//Unable to move to that location, print error and usage
		fclose(dll);
		output("Error: Unable to seek to the proper position in '");
		output(dll_name);
		output("'!\n");
		usage();
		return 1;
	}
	/*Write the bytes at that location*/
	if(fwrite(buffer, 16, 1, dll) != 1)
	{
		/*Unable to read enough bytes, print error and usage*/
		fclose(dll);
		output("Error: Unable to write the modifications to '");
		output(dll_name);
		output("'!\n");
		usage();
		return 1;
	}
	fclose(dll);
	output("Successfully patched file '");
	output(dll_name);
	output("'!\n");
	return 0;
}

int main(int argc, char *argv[])
{
	output("Copyright 2006 Extrarius (extrarius@extrarius.name)\n");
	output("Redistribution and use without modification are permitted.\n\n");

	output("This program is meant to fix the sourge 'engine.dll' used by\n");
	output("'Vampire: The Masquerade Bloodlines'(TM) so that it will work\n");
	output("on machines with more than 2GB of RAM.\n\n");
	output("The game must be patched to version 1.20 for this to work.\n\n");
	output("There is no guarantee that it will function correctly.\n\n");
	output("Read the included file 'license.txt'\n\n");
	
	if(argc != 2)
	{
		usage();
		return 1;
	}
	return patch_vamp(argv[1]);
}